Type: Package
Package: directadjusting
Title: Directly Adjusted Estimates
Version: 0.7.0
Authors@R: 
    person("Joonas", "Miettinen", , "joonas.miettinen@cancer.fi", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-8624-6754"))
Description: Compute estimates and confidence intervals of weighted
    averages quickly and easily. Weighted averages are computed using
    data.table for speed. Confidence intervals are approximated using the
    delta method with either using known formulae or via algorithmic or
    numerical derivation.
License: MIT + file LICENSE
URL: https://github.com/FinnishCancerRegistry/directadjusting
BugReports: https://github.com/FinnishCancerRegistry/directadjusting/issues
Depends: R (>= 2.10)
Imports: data.table, stats
Config/roxygen2/version: 8.0.0
Encoding: UTF-8
Language: en-GB
NeedsCompilation: no
Packaged: 2026-07-14 09:04:46 UTC; joonas.miettinen
Author: Joonas Miettinen [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-8624-6754>)
Maintainer: Joonas Miettinen <joonas.miettinen@cancer.fi>
Repository: CRAN
Date/Publication: 2026-07-14 09:50:02 UTC
Built: R 4.5.2; ; 2026-07-14 13:23:33 UTC; unix
