Type: Package
Package: directadjusting
Title: Directly Adjusted Estimates
Version: 0.6.1
Authors@R: 
    person("Joonas", "Miettinen", , "joonas.miettinen@cancer.fi", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0001-8624-6754"))
Description: Compute estimates and confidence intervals of weighted
    averages quickly and easily. Weighted averages are computed using
    data.table for speed. Confidence intervals are approximated using the
    delta method with either using known formulae or via algorithmic or
    numerical integration.
License: MIT + file LICENSE
URL: https://github.com/FinnishCancerRegistry/directadjusting/
BugReports: https://github.com/FinnishCancerRegistry/directadjusting/issues
Depends: R (>= 2.10)
Imports: data.table, stats
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-02 09:01:43 UTC; joonas.miettinen
Author: Joonas Miettinen [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-8624-6754>)
Maintainer: Joonas Miettinen <joonas.miettinen@cancer.fi>
Repository: CRAN
Date/Publication: 2026-02-04 19:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 16:20:19 UTC; unix
